<?php
/*
 * GriefPrevention
 * Bukkit/Spigot Plugin
 * http://dev.bukkit.org/bukkit-plugins/grief-prevention/
 * https://www.spigotmc.org/resources/griefprevention.1884/
 */
class Minecraft_mod_griefprevention {
	private $server			= array();			# Current server settings loaded
	
	function __construct($server=array()) {
		$this->server = $server;
	}
	
	function option($field, $default=false){
		if(isset($this->server['metadata'][$field])){
			return $this->server['metadata'][$field];
		} else if(isset($this->server[$field])){
			return $this->server[$field];
		}

		return $default;
	}
		
	/* ============================================================
	 * CHAT
	 * ------------------------------------------------------------
	 * Used to parse each line of the log files
	 * Should return either false or a structure array
	 * false in the event of a non-match
	 * structure array depending on properties
	 */

	public $regexp_name		= '[A-z0-9_]{2,16}';
	public $regexp_ip		= '[0-9\.]+';
	public $regexp_coords	= '-?[0-9\.]+,\s*-?[0-9\.]+,\s*-?[0-9\.]+';
	
	private $appendable		= false;
	
	function parse_log_line($line){

		if(strpos($line, 'GriefPrevention:')!=false && preg_match('/\[([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})\]\s+\[([^\]]+)\]:\s+(.*?)$/i', $line, $matches)){
			$this->appendable = false;

			$time	= trim($matches[1]);
			$type	= strtolower(trim($matches[2]));
			$text	= trim($matches[3]);


			#[14:23:29] [Async Chat Thread - #5/INFO]: GriefPrevention: Muted gibberish.
			#[14:26:10] [Server thread/INFO]: GriefPrevention: DarthWillieDSTAR placed a sign @ world: x2865, z242;Willie's;House
			#[16:10:14] [Server thread/INFO]: GriefPrevention: JebusD placed suspicious WATER_BUCKET @ world: x-593, z2754
			#[17:46:32] [Server thread/INFO]: GriefPrevention: Jacko_09 placed suspicious LAVA_BUCKET @ world: x4386, z3630
			#[19:07:12] [Server thread/INFO]: GriefPrevention: atempting chest claim 4
			if(preg_match('/GriefPrevention:\s+(' . $this->regexp_name . ')\s+placed/is', $text, $matches)){
				#echo '<pre>' . print_r($matches, true) . '</pre>';
				$user = $matches[1];
				return array(
					'time'		=> $time,
					'type'		=> 'info',
					'user'		=> trim($user),
					'text'		=> trim($text),
				);
			}
		}
		
		return false;
	}
}
