<?php
# TODO Abstract to be more db agnostic
class db {
	public $dbh = null;	# TODO make private

	function connect($config){
		// Create Database connection
		try {
			$this->conn = new PDO('mysql:host=' . $config['host'] . ';dbname=' . $config['name'], $config['user'], $config['pass']);
			$this->conn->setAttribute( PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION );
		} catch (PDOException $e) {
			system_message($e->getMessage());
		}
	}

	function insert($table, $fields){
		$columns_string = implode(',', array_keys($fields));
		$values_string = implode(',', array_fill(0, count($fields), '?'));
		$sql = "INSERT INTO " . $table . " (" . $columns_string . ") VALUES (" . $values_string . ")";
		$values = array_values($fields);
		
		# Run
		$sth = $this->conn->prepare($sql);
		if (!$sth) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
		$result = $sth->execute($values);
		if (!$result) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
	}

	function insert_ignore($table, $fields){
		$columns_string = implode(',', array_keys($fields));
		$values_string = implode(',', array_fill(0, count($fields), '?'));
		$sql = "INSERT IGNORE INTO " . $table . " (" . $columns_string . ") VALUES (" . $values_string . ")";
		$values = array_values($fields);
		
		# Run
		$sth = $this->conn->prepare($sql);
		if (!$sth) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
		$result = $sth->execute($values);
		if (!$result) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
	}

	function insert_update($table, $fields, $updates){
		$columns_string = implode(',', array_keys($fields));
		$values_string = implode(',', array_fill(0, count($fields), '?'));
		$sql = "INSERT INTO " . $table . " (" . $columns_string . ") VALUES (" . $values_string . ")
			ON DUPLICATE KEY UPDATE ";

		$i = 0;
		
		$values = array_values($fields);

		foreach($updates as $field=>$value){
			if($i++>0){$sql .= ',';}
			if($field===true || $field==$value){
				$sql .= $field . '=VALUES(' . $field . ')';
			} else {
				$sql .= $field . '=?';
				$values[] = $value;
			}
		}

		
		# Run
		$sth = $this->conn->prepare($sql);
		if (!$sth) {
			echo "\nPDO::errorInfo():\n";
			echo '<pre>' . $sql . '</pre>';;
			print_r($sth->errorInfo());
		}
		$result = $sth->execute($values);
		if (!$result) {
			echo "\nPDO::errorInfo():\n";
			echo '<pre>' . $sql . '</pre>';;
			print_r($sth->errorInfo());
		}
	}

	function update($table, $fields, $criteria){
		$columns_string = implode('=?,', array_keys($fields));
		$criteria_string = implode('=? AND ', array_keys($criteria));
		
		$sql = "UPDATE " . $table . " SET " . $columns_string . "=? WHERE " . $criteria_string . '=?';

		$values = array_merge(array_values($fields), array_values($criteria));
		
		# Run
		$sth = $this->conn->prepare($sql);
		if (!$sth) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
		$result = $sth->execute($values);
		if (!$result) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
	}

	function delete($table, $criteria){
		$criteria_string = implode('=? AND ', array_keys($criteria));
		
		$sql = "DELETE FROM " . $table . " WHERE " . $criteria_string . '=?';
		
		$values = array_values($criteria);
		
		# Run
		$sth = $this->conn->prepare($sql);
		if (!$sth) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
		$result = $sth->execute($values);
		if (!$result) {
			echo "\nPDO::errorInfo():\n";
			print_r($sth->errorInfo());
		}
	}
	
	function query($sql){
		try {
			$result = $this->conn->query($sql, PDO::FETCH_ASSOC);
			if($result===false){
				return array();
			} else {
				return $result;
			}
		} catch (PDOException $err) {
			print_r($err);
			exit;
		}
	}
	
	function fetch($sql){
		try {
			$sth = $this->conn->prepare($sql);
			$sth->execute();
			$result = $sth->fetch(PDO::FETCH_ASSOC);
			if($result===false){
				return array();
			} else {
				return $result;
			}
		} catch (PDOException $err) {
			print_r($err);
			exit;
		}
	}
	
	function escape($str){
		return $this->conn->quote($str);
	}
	
	function escape_like($str){
		return substr($this->conn->quote($str), 1, -1);
	}
}

