/* ============================================================
 * Globals
 */
dateFormatStr = {
	daysFull: ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday', 'Sunday'],
	daysAbbr: ['Sun','Mon','Tue','Wed','Thur','Fri','Sat', 'Sun'],
	daysInit: ['S','M','T','W','T','F','S', 'S'],
	monthsFull: ['January','February','March','April','May','June','July','August','September','October','November','December'],
	monthsAbbr: ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']
}

/* ============================================================
 * Date Helpers
 */
dateHelpers = {}
dateHelpers.padDateNum = function(n){return n<10 ? '0'+n : n}
dateHelpers.getMonday = function(d) {
	if(typeof(d)=='string'){
		d = new Date(d);
	}
	var day = d.getDay(),
		diff = d.getDate() - day + (day == 0 ? -6:1); // adjust when day is sunday
	return new Date(d.setDate(diff));
}

dateHelpers.getYmd = function(d){
	if(typeof(d)=='string'){
		d = new Date(d);
	} else if(typeof(d)=='object'){
		d = new Date(d);
	} else {
		d = new Date();
	}
	
	if(typeof(d)=='object'){
		return d.getFullYear() + '-' + this.padDateNum(d.getMonth()+1) + '-' + this.padDateNum(d.getDate());
	} 
	
	return '';
}

dateHelpers.getdmY = function(d){
	if(typeof(d)=='string'){
		d = new Date(d);
	}
	
	if(typeof(d)=='object'){
		return this.padDateNum(d.getDate()) + '/' + this.padDateNum(d.getMonth()+1) + '/' + d.getFullYear();
	} 
	
	return '';
}

dateHelpers.getDateFromPicker = function (){
	var val = $('.page-calendar-date-select').val(); // todo: Date picker integration
	if(val!=''){
		return new Date(val);
	}
	
	return new Date();	// This function must return a date
}


dateHelpers.strtotime = function(d){
	var a, b, c;
	if(d.match(/[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}/ig)){
		a = d.split('-');
		d = new Date();
		d.setFullYear(a[0]);
		d.setMonth(a[1]-1);
		d.setDate(a[2]);
		d.setHours(0);
		d.setMinutes(0);
		d.setSeconds(0);
		return d;
	} else if(d.match(/[0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4}/ig)){
		a = d.split('/');
		d = new Date();
		d.setFullYear(a[2]);
		d.setMonth(a[1]-1);
		d.setDate(a[0]);
		d.setHours(0);
		d.setMinutes(0);
		d.setSeconds(0);
		return d;
	}
	
    if (arguments.length <= 3)
        throw new Error("dateHelpers.strtotime Doesn't know that format");
}

dateHelpers.getToday = function(){
	var d = new Date();
	d.setHours(0);
	d.setMinutes(0);
	d.setSeconds(0);
	return d;
}

/* ============================================================
 * Handle Bar Helpers
 */
/*
function: compare
{{#compare unicorns < ponies }}
I knew it, unicorns are just low-quality ponies!
{{/compare}}
*/
Handlebars.registerHelper('compare', function(lvalue, operator, rvalue, options) {
    if (arguments.length <= 3)
        throw new Error("Handlerbars Helper 'compare' needs 3 parameters");

    var operators = {
        '==':       function(l,r) { return l == r; },
        '===':      function(l,r) { return l === r; },
        '!=':       function(l,r) { return l != r; },
        '<':        function(l,r) { return l < r; },
        '>':        function(l,r) { return l > r; },
        '<=':       function(l,r) { return l <= r; },
        '>=':       function(l,r) { return l >= r; },
        '%':       function(l,r) {
			if(l==0){
				return false;
			} else {
				return (l+1) % r;
			}
		},
        'typeof':   function(l,r) { return typeof l == r; },
        'in':   function(l,r) { 
			if(typeof(r)=='string'){
				r = r.split(',');
			}
			
			for(var i=0;i<r.length;i++){
				if(r[i]==l){
					return true;
				}
			}
			
			return false;
		}
    };

    if (!operators[operator])
        throw new Error("Handlerbars Helper 'compare' doesn't know the operator "+operator);

    var result = operators[operator](lvalue,rvalue);

    if( result ) {
        return options.fn(this);
    } else {
        return options.inverse(this);
    }
});

/*
function: formatDate
{{dateFormatStr date format }}
*/
Handlebars.registerHelper('formatDate', function(format, date, options) {
	// Default Format
	if(typeof(format)!='string'){
		format = 'Y-m-d H:i:s';
	}
	var d, escape=false;
	var char = str = leadDay = leadMonth = leadHour = leadMinute = leadSecond = '';
	
	// Date object, default to now
	if(typeof(date)=='object' && date!==null && typeof(date.getMonth)=== 'function'){
		d = date;
	} else if(typeof(date)=='undefined' || date==null || date==''){
		d = new Date();
	} else if(typeof(date)=='integer') {
		d = new Date();
		d.setTime(date);
	} else {
		d = new Date(date);
	}
	
	// Loop over format
	for (var i = 0; i < format.length; i++) {
		char = (format.charAt(i));
		if(escape === true){
			escape = false;
			str += char;
		} else if(char=='\\'){
			escape = true;
		} else {
			switch (char) {
				// Day
				case 'd':
					if(d.getDate()<=9){leadDay = '0'}
					str += leadDay + d.getDate();
					break;
				case 'D':
					str += dateFormatStr.daysAbbr[d.getDay()];
					break;
				case 'j':
					str += d.getDate();
					break;
				case 'l':
					str += dateFormatStr.daysFull[d.getDay()];
					break;
	
				// Week

				// Month
				case 'F':
					str += dateFormatStr.monthsFull[d.getMonth()];
					break;
				case 'M':
					str += dateFormatStr.monthsAbbr[d.getMonth()];
					break;
				case 'm':
					if(d.getMonth()<9){leadMonth = '0';}
					str += leadMonth + (d.getMonth()+1);
					break;

				// Year
				case 'Y':
					str += d.getFullYear();
					break;

				// Time
				case 'H':
					str += d.getHours();
					break;
					
				case 'h':
					if(d.getHours()>12){
						str += d.getHours() - 12;
					} else {
						str += d.getHours();
					}
					break;

				case 'i':
					if(d.getMinutes()<=9){leadDay = '0';}
					str += leadDay + d.getMinutes();
					break;
					
				// Meridiem
				case 'a':
					if(d.getHours()>=12){
						str += 'am';
					} else {
						str += 'pm';
					}
					break;

				case 'A':
					if(d.getHours()>=12){
						str += 'AM';
					} else {
						str += 'PM';
					}
					break;
				
				// Default Text
				default: 
					str += char;
					break;
			}
		}
	}
	
	// Done
	return new Handlebars.SafeString(str);
});

/*
function: substr
{{substr str , start , length }}
*/
Handlebars.registerHelper('substr', function(str, start, length, options) {
	if(length>0){} else {length = str.length;}
	return str.substr(start, length);
});

/*
function: toJSON
{{toJSON object }}
*/
Handlebars.registerHelper('toJSON', function(object){
	return new Handlebars.SafeString(JSON.stringify(object));
});

/*
function: math
{{math lvalue, operator, rvalue}}
*/
Handlebars.registerHelper("math", function(lvalue, operator, rvalue, options) {
    lvalue = parseFloat(lvalue);
    rvalue = parseFloat(rvalue);
        
    return {
        "+": lvalue + rvalue,
        "-": lvalue - rvalue,
        "*": lvalue * rvalue,
        "/": lvalue / rvalue,
        "%": lvalue % rvalue
    }[operator];
});